import socket
import wolfssl
import os

def run_client(file_path):
    ca_cert = "../certificates/ca-cert.pem"
    print("Wählen Sie einen Verschlüsselungsalgorithmus:")
    print("1: AES 128 GCM (sicher)")
    print("3: ChaCha20-Poly1305 (sicher)")
    print("4: RC4 (unsicher)")
    print("5: 3DES (unsicher)")
    
    choice_cipher = input("Geben Sie die Nummer des gewünschten Verschlüsselungsalgorithmus ein: ")

    if choice_cipher == "1":
        cipher_suite = "TLS_AES_128_GCM_SHA256"
    elif choice_cipher == "2":
        cipher_suite = "TLS_CHACHA20_POLY1305_SHA256"
    elif choice_cipher == "3":
        cipher_suite = "TLS_RSA_WITH_RC4_128_SHA"
    elif choice_cipher == "4":
        cipher_suite = "TLS_RSA_WITH_3DES_EDE_CBC_SHA"
    else:
        print("Ungültige Auswahl, Standard-AES 128 GCM wird verwendet.")
        cipher_suite = "TLS_AES_128_GCM_SHA256"

    context = wolfssl.SSLContext(wolfssl.PROTOCOL_TLSv1_2)
    context.load_verify_locations(ca_cert)

    try:
        context.set_ciphers(cipher_suite)
        print(f"Cipher Suite '{cipher_suite}' erfolgreich gesetzt!")
    except Exception as e:
        print(f"Fehler beim Setzen der Cipher Suite: {e}")
        return

    try:
        print("Verbindung zum Server wird aufgebaut...")
        with socket.create_connection(("192.168.125.101", 5000)) as sock:
            with context.wrap_socket(sock, server_hostname="server") as ssl_sock:
                print("Mit dem Server verbunden!")

                with open(file_path, "rb") as f:
                    while chunk := f.read(1024):
                        ssl_sock.sendall(chunk)
                print(f"Datei '{file_path}' erfolgreich an den Server gesendet.")

    except Exception as e:
        print(f"Fehler bei der Verbindung zum Server: {e}")

if __name__ == "__main__":
    file_to_send = "../testData/test.txt"
    if not os.path.exists(file_to_send):
        print(f"Die Datei '{file_to_send}' wurde nicht gefunden!")
    else:
        run_client(file_to_send)
