import socket
import wolfssl

def run_server():
    cert_file = "../certificates/server-cert.pem"
    key_file = "../certificates/server-key.pem"

    # Cipher Suite Auswahl
    print("Wählen Sie einen Verschlüsselungsalgorithmus:")
    print("1: AES 128 GCM (sicher)")
    print("2: ChaCha20 Poly1305 (sicher)")
    print("3: RC4 (unsicher)")
    print("4: 3DES (unsicher)")
    choice_cipher = input("Geben Sie die Nummer des gewünschten Verschlüsselungsalgorithmus ein: ")

    if choice_cipher == "1":
        cipher_suite = "TLS_AES_128_GCM_SHA256"
    elif choice_cipher == "2":
        cipher_suite = "TLS_CHACHA20_POLY1305_SHA256"
    elif choice_cipher == "3":
        cipher_suite = "TLS_RSA_WITH_RC4_128_SHA"
    elif choice_cipher == "4":
        cipher_suite = "TLS_RSA_WITH_3DES_EDE_CBC_SHA"
    else:
        print("Ungültige Auswahl, Standard-AES 128 GCM wird verwendet.")
        cipher_suite = "TLS_AES_128_GCM_SHA256"

    context = wolfssl.SSLContext(wolfssl.PROTOCOL_TLSv1_2)
    context.load_cert_chain(certfile=cert_file, keyfile=key_file)
    context.set_ciphers(cipher_suite)

    # Server-Socket erstellen
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
        sock.bind(("0.0.0.0", 5000))  
        sock.listen(5)
        print(f"Server läuft auf Port 5000 mit Cipher Suite: {cipher_suite}...")

        while True:
            log("Warten auf eingehende Verbindungen...")
            client_sock, addr = sock.accept()
            log(f"Verbindung akzeptiert von {addr}!")

            try:
                ssl_sock = context.wrap_socket(client_sock, server_side=True)
                log("SSL/TLS-Verbindung erfolgreich hergestellt.")
                
                with open("received_file.txt", "wb") as f:
                    while True:
                        data = ssl_sock.recv(1024)
                        if not data:
                            break
                        f.write(data)
                log("Datei erfolgreich empfangen und gespeichert als 'received_file.txt'")

            except Exception as e:
                log(f"Fehler bei der Verarbeitung der Verbindung: {e}")

            finally:
                ssl_sock.close()
                client_sock.close()

def log(message):
    print(message)  # Hier können Sie auch in eine Log-Datei schreiben.

if __name__ == "__main__":
    run_server()
